import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

public class DrawingController extends Controller {

	//---------------------------------------------------
	//Ivars	

	private Drawing model;
	private View view;
	private List<Point> points;	
	private Drawing.Action action;
	
	//---------------------------------------------------
	//Constructors	

	public DrawingController(Drawing aModel, View aView) {
		super();
		this.model = aModel;
		this.view = aView;
		this.points = null;
		this.clearAction();
	}

	//---------------------------------------------------
	//Getters

	public Drawing getModel() {
		return model;
	}
	
	//---------------------------------------------------
	//Overrides (not needed) -- TODO -- make overrides optional	
	
	@Override
	public void refresh() {}
	
	@Override	
	public void apply() {}	

	//---------------------------------------------------
	//Hooking Up Events	

	protected void hookUpEvents() {
		((JButton) this.get("addLineButton")).addActionListener(evt -> this.clickedAddLine());
		((JButton) this.get("addPolylineButton")).addActionListener(evt -> this.clickedAddPolyline());		
		((JButton) this.get("clearButton")).addActionListener(evt -> this.clickedClear());
		((JPanel) this.get("graphicsPane")).addMouseListener(new DrawingMouseListener(this));		
	}

	//---------------------------------------------------
	//Framework Methods
	
	@Override
	public View getView() {
		return this.view;
	}
	
	//---------------------------------------------------
	//Instance Methods
	
	private void clearAction() {
		this.action = Drawing.Action.NO_ACTION;		
	}
	
	JLabel getMessageLabel() {
		return (JLabel)this.get("message");
	}	
	
	//---------------------------------------------------
	//Action Handlers

	private void clickedClear() {
		this.doAction(Drawing.Action.CLEAR);
	}
	
	private void doAction(Drawing.Action anAction) {
		DrawingView v = (DrawingView)this.view;
		this.action = anAction;
		try {
			v.setActionRunning(true);
			v.getDrawingArea().repaint();	
		} finally {
		}		
	}

	public void drawOn(Graphics g) {
		if (this.isAcquiringPoints()) {
			this.model.addDrawable(this.action, this.points);
			this.postAddPolyline();
		}
		else
			this.model.doAction(this.action);
		this.model.drawOn(g);		
		this.clearAction();		
	}
		
	//---------------------------------------------------
	//User-Action Handlers
	
	private void clickedAddLine() {
		this.action = Drawing.Action.ADD_LINE;
		this.startAddingDrawable("Draw line by clicking on start and end points");		
	}	
	
	private void clickedAddPolyline() {
		this.action = Drawing.Action.ADD_POLYLINE;
		this.startAddingDrawable("Draw polyline by left-clicking mouse, right-click to finish");
	}	
	
	private void startAddingDrawable(String userMessage) {
		this.getMessageLabel().setText(userMessage);
		this.points = new ArrayList<>();
	}
	
	private void finishAddingDrawable() {
		this.doAction(this.action);		
	}		
	
	private void finishPolyline() {
		this.doAction(Drawing.Action.ADD_POLYLINE);
	}
	
	private boolean isAcquiringPoints() {
		return this.points != null;
	}

	public void clickedPointAt(Point p) {
		if (this.isAcquiringPoints()) {
			this.points.add(p);
		if (this.points.size() == model.expectedPointCountFor(this.action))
			this.finishAddingDrawable();
		}
	}

	public void rightClicked() {
		if (this.isAcquiringPoints())
			this.finishPolyline();
	}
	
	/** Must be called at end of polyline action */
	private void postAddPolyline() {
		this.points = null;
		this.getMessageLabel().setText(" ");
	}	
		
}

//======================================================================

class DrawingMouseListener implements MouseListener {
	
	private DrawingController controller;
	
	public DrawingMouseListener(DrawingController aController) {
		super();
		this.controller = aController;
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		//nop (this handler misses clicks -- use pressed or released)
	}

	@Override
	public void mousePressed(MouseEvent e) {
		//nop
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		Point p = new Point(e.getX(), e.getY());
		if (SwingUtilities.isLeftMouseButton(e))
			this.controller.clickedPointAt(p);
		else if (SwingUtilities.isRightMouseButton(e))
			this.controller.rightClicked();
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		//nop
	}

	@Override
	public void mouseExited(MouseEvent e) {
		//nop
	}
}
